package
Version:
v0.3.4-0...-1eafea6
Opens a new window with list of versions in this module.
Published: Jul 24, 2026
License: MIT
Opens a new window with license information.
Imports: 1
Opens a new window with list of imports.
Imported by: 0
Opens a new window with list of known importers.
Documentation
¶
Package lazy provides one-shot lazy initialization of a value.
type Lazy[T any] struct {
}
Lazy provides one-shot lazy initialization of a value T.
New creates a new Lazy which when initialized for the first time calls the initialization function provided.
Source Files
¶
Click to show internal directories.
Click to hide internal directories.